% Model file sat.m
% This M-file contains the model of the open-loop satellite 
% with reaction wheel described in Appendix A of "Feedback 
% Control Problems using MATLAB and the Control System 
% Toolbox" by D. K. Frederick and J. H. Chow. 
% The file allows the user to get 4 transfer functions or 
% the state-space model.
%
%   states are: x1 = pointing angle (rad)
%		x2 = angular velocity (rad/sec)
%		x3 = wheel speed (rad/sec)
%   inputs are: u = motor torque (Newton-meters)
%		w = external torque (N-m) can be
%		     either disturbance or gas jets
%  outputs are: theta = pointing angle (degrees)
%		omega = wheel speed (rev/min)
%    state-space equations used:
%	xdot = A * x + B * u + G * w
%	  y  =  C * x

%%%%%%%%%%%%%%%%%%%%%%% sat.m %%%%%%%%%%%%%%%%%%%%%%%
%       Feedback Control Problems with MATLAB 
%           and the Control System Toolbox
%      D. K. Frederick and J. H. Chow, May 1999
%             last revision: 5/23/99 HS
%----------------------------------------------------
clear
format short e	% use floating point
%
% Satellite & reaction wheel, no controller
%
Js  = 13.6;	% satellite moment of inertia (kg m^2)
Jw  =  1.36e-4;	% wheel moment of inertia (kg m^2)
B   =  1.01e-6;	% frictional coefficient (N m /(rad/sec))
Jeq = Js*Jw/(Js + Jw);	% equivalent moment of inertia (kg m^2)
%
A = [0  1  0     ;
     0  0  B/Js  ;
     0  0 -B/Jeq ];
%
B = [  0     ;		% gains for motor torque (internal)
      -1/Js  ;
      1/Jeq ];
%
G = [  0  ;		% gains for external inputs
     1/Js ;
    -1/Js ];
%----- outputs are in degrees & rpm so need to convert
C = [ 180/pi     0        0   ;
	0        0   60/(2*pi) ];
%
D = zeros(2,1);
%
%----- transfer function matrix for motor input
[num_mtr,den_mtr] = ss2tf(A,B,C,D);
%---- zero out any very small elements in numerator
%	num_mtr(1,2) & num_mtr(1,4) --> 0
num_mtr = small20(num_mtr,1.0e-10);
%----- transfer function matrix for disturbance torque
[num_dst,den_dst] = ss2tf(A,G,C,D);
%---- zero out any very small elements in numerator
%	num_dst(1,2) & num_dst(1,4) --> 0
num_dst = small20(num_dst,1.0e-10);
%---- define time vector from 0 to 10 minutes
t = [0:2:600]';
disp('  ')
disp('t = time vector from 0 to 10 minutes')
disp('.....You can use it with step, impulse, & lsim commands.')
disp('  ')
disp('*******>');pause
% may want to restore these for later use
save sat.mat A B C D G num_mtr den_mtr num_dst den_dst t
%------------- menu to select what you want ----------
condition = -1;	% force menu to appear
while (condition < 1) | (condition > 6),
  disp(' 1 <= G(s): motor torque --> pointing angle')
  disp(' 2 <= G(s): motor torque --> wheel speed')
  disp(' 3 <= G(s): external torque --> pointing angle')
  disp(' 4 <= G(s): external torque --> wheel speed')
  disp(' 5 <= state-space model')
  disp(' 6 <= create file sat.mat')
%
  disp('Enter 1...5 to select a model,')
  condition = input('   6 for file, or 0 to terminate => ');

if condition == 0,
  disp('Terminating satellite model'), 
  break
end
%--------------------------------------
%
if condition == 1,
  disp('G(s): motor torque --> pointing angle')
  numG = num_mtr(1,:)
  denG = den_mtr
  G = tf(numG,denG)
  %----- plot a step response
  taum = 1.0e-4*ones(size(t));
  y = lsim(G,taum,t);
  plot(t,y);grid
  ylabel('degrees');xlabel('seconds')
  title('ol_sat: pointing angle due to step motor torque of 1.0e-4 N-m')
  keyboard
  load sat.mat	% restore state-space matrices & G(s)
  disp('end of motor torque --> pointing angle')
  condition = -1;	% force menu to appear
end	% of condition = 1, so return to menu
%-------------------------------------
%
if condition == 2,
  disp('G(s:) motor torque --> wheel speed')
  numG = num_mtr(2,:)
  denG = den_mtr
  G = tf(numG,denG)
  taum = 1.0e-4*ones(size(t));
  y = lsim(G,taum,t);
  plot(t,y);grid
  ylabel('wheel speed (rpm)');xlabel('seconds')
  title('ol_sat: wheel speed due to step motor torque of 1.0e-4 N-m')
  keyboard
  load sat.mat	% restore state-space matrices & G(s)
  disp('end of motor torque --> wheel speed')
  condition = -1;	% force menu to appear
end	% of condition = 2, so return to menu
%-------------------------------------
%
if condition == 3,
  disp('G(s): external torque --> pointing angle')
  numG = num_dst(1,:)
  denG = den_dst
  G = tf(numG,denG)
  y = 0.01*impulse(G,t);
  plot(t,y);grid
  ylabel('degrees');xlabel('seconds')
  title('ol_sat: pointing angle due to impulsive dist. torque of 0.01 N-m-s')
  keyboard
  load sat.mat	% restore state-space matrices & G(s)
  disp('end of external torque --> pointing angle')
  condition = -1;	% force menu to appear
end	% of condition = 3, so return to menu
%-------------------------------------
%
if condition == 4,
  disp('G(s): external torque --> wheel speed')
  numG = num_dst(2,:)
  denG = den_dst
  G = tf(numG,denG)
  y = 0.01*impulse(G,t);
  plot(t,y);grid
  ylabel('rpm');xlabel('seconds')
  title('ol_sat: wheel speed due to impulsive dist. torque of 0.01 N-m-s')
  keyboard
  load sat.mat	% restore state-space matrices & G(s)
  disp('end of external torque --> pointing angle')
  condition = -1;	% force menu to appear
end	% of condition = 4, so return to menu
%-------------------------------------
if condition == 5,
  disp('state-space matrices for:')	
  disp('')	
  disp('   xdot = A * x + B * u + G * w')
  disp('     y  =  x')
  disp('  states are:')
  disp('        x1 = pointing angle (rad)')
  disp('        x2 = angular velocity (rad/sec)')
  disp('        x3 = wheel speed (rad/sec)')
  disp(' inputs are:')
  disp('         u = motor torque ')
  disp('         w = external torque ')
  disp('')
  A
  disp('*****>');pause
  disp('gains for motor torque input')
  B
  disp('*****>');pause
  disp('gains for degrees & rev/minute outputs')
  C
  disp('*****>');pause
  D
  disp('*****>');pause  
  disp('gains for external torque input')
  G
  keyboard
  load sat.mat	% restore state-space matrices & G(s)
  disp('end of state-space form'); disp('  ')
  condition =-1;	% force menu to appear
end	%  of condition = 5, so return to menu
%-------
if condition == 6,
   numG1 = num_mtr(1,:);
   denG1 = den_mtr; 
   numG2 = num_mtr(2,:);
   denG2 = den_mtr;
   numG3 = num_dst(1,:);
   denG3 = den_dst;
   numG4 = num_dst(2,:);
   denG4 = den_dst;
   save sat.mat numG1 denG1 numG2 denG2 ...
         	numG3 denG3 numG4 denG4 A B C D G t 
  disp('file sat.mat has been saved')
  disp('*******>');pause
  condition = -1;
end
%------
end		% of while
disp('TF form of model in numG & denG if used options 1....4')
disp('State-space model in A, B, C, D if used option 5')
disp('All transfer functions and state-space matrices are')
disp('.....defined if used option 6.')
%%%%%%%%%%%%%%%%%%%% end of sat.m %%%%%%%%%%%%%%%%%%%